/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */
package com.yuandian.dataflow.controller;

import java.io.Serializable;
import java.time.Instant;
import java.util.ArrayList;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.alipay.sofa.jraft.rpc.RpcContext;
import com.alipay.sofa.jraft.rpc.RpcProcessor;
import com.google.protobuf.Any;
import com.yuandian.dataflow.statemachine.StateServerFactory;
import com.yuandian.dataflow.statemachine.SyncClosure;
import com.yuandian.dataflow.statemachine.rpc.RaftResponse;
import com.yuandian.dataflow.statemachine.rpc.SyncStateProcessor;
import com.yuandian.dataflow.statemachine.rpc.SyncWorkerStateProcessor;
import com.yuandian.dataflow.statemachine.rpc.annotations.ProcessorRaft;
import com.yuandian.dataflow.statemachine.state.State;

import lombok.Getter;
import lombok.Setter;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */

@Slf4j
@ProcessorRaft
public class PacketsProcessor implements RpcProcessor<PacketsProcessor.PacketsRequest> {

    @Setter
    @Getter
    public static class PacketsRequest implements Serializable {
        private ArrayList<Any> packets = new ArrayList<>();
    }

    @Override
    public void handleRequest(RpcContext rpcCtx, PacketsRequest request) {
            // StateServerFactory.getStateServer().updateFsmStateAsync(s, onCompleted);
            log.info("request.packets.size(): {}",request.packets.size());
            var resp = new RaftResponse();
            resp.setMsg(rpcCtx.getRemoteAddress());
            resp.setSuccess(true);
 
            var ss = StateServerFactory.getStateServer();
 
            ss.useFsmStateNotLock((state)->{
                
                var work = state.getWorkers().get( ss.getCluster().getServerId());
 
              
                work.setTaskQueueSize( work.getTaskQueueSize() - request.packets.size());
                work.setUpdateAt(Instant.now());
                log.debug("workerState taskQueueSize: {} psize: {}", work.getTaskQueueSize(), request.packets.size());
                if(!ss.isLeader()) {
                    var requestUpdateState = new SyncWorkerStateProcessor.RequestWorkerState();
                    requestUpdateState.setWorkerState(work); 
                    log.info("转发 {}", work);
                    try {
                        ss.getRpcClient().invokeAsync(StateServerFactory.getNode().getLeaderId().getEndpoint(),
                        requestUpdateState, new InvokeCallback() {
                        @Override
                        public void complete(Object result, Throwable err) {
                            if (err != null) {
                                err.printStackTrace();
                            } else {
                                log.debug("转发 update WorkerState {}", result);
                            }
                            rpcCtx.sendResponse(resp);
                        }}, 5000);
                    } catch (InterruptedException | RemotingException e) {
                        e.printStackTrace();
                    }  
                    return;
                }

                StateServerFactory.getStateServer().applyWorkerState(work, new SyncClosure<State>() {
                    @Override
                    public void run(Status status) {
                        log.debug("finsh tasks size {}, size: {}", status, request.packets.size());
                        rpcCtx.sendResponse(resp);
                    }
                });
            });
            

    }

    @Override
    public String interest() {
        return PacketsRequest.class.getName();
    }
}
